******************************************************************;
* The file ALL1 is a SAS version of the spreadsheet reported on  *;
* the Datacolada website.                                        *;
* The variable names should be self-explanatory -- for example,  *;
* car1_ins is the mileage for car 1 from the insurance files,    *;
* (i.e., time 1, from column C in the spreadsheet                *;
* car1_rsp is the mileage for car 1 from the respondent          *;
* (i.e., time 2, from column D in the spreadsheet).              *;
******************************************************************; 


******************************************************************; 
* This illustrates how to read the CSV version of the raw data   *;
* from Shu into a temporary dataset named ALL1.  This can be     *;
* used to verify that ALL1 is consistent with the raw data.      *;
* You will need to replace the file location with the full path  *;
* name on your computer.                                         *;
******************************************************************;

filename reffile 'insert file location.csv';
proc import datafile=reffile
     dbms=csv
	 out=all1;
run;


******************************************************************;
* The libname statement points to a folder location on my        *;
* desktop, and should be replaced with the folder location on    *;
* yours.                                                         *;
*                                                                *;
******************************************************************;

libname Columbo 'C:\Users\samsa001\Desktop\Columbo';
run;

******************************************************************;
* Listing of the variables.                                      *;
******************************************************************;


proc contents position data=Columbo.all1;
  title 'study variables';
run;



******************************************************************;
* Print the first 20 records.                                    *;
******************************************************************; 

proc print data=Columbo.all1(obs=20);
  title 'analysis file';
  title2 '20 records';
run; 



******************************************************************;
* Verify that K-M are correctly derived.                         *;
******************************************************************;

data test;
  set Columbo.all1;
  test_k=mean(car1_ins,car2_ins,car3_ins,car4_ins)-avg_ins;
  test_l=mean(car1_rsp,car2_rsp,car3_rsp,car4_rsp)-avg_rsp;
  test_m=(avg_rsp-avg_ins)-avg_miles;

proc freq data=test;
  tables test_k test_l test_m;
  title 'question 2';
  title2 'all frequencies should be 0, apart from rounding error';
run;


******************************************************************;
* Verify that the reported between-group comparison is           *;
* correctly reported.                                            *;
******************************************************************;


******************************************************************;
* If the randomization was successful, the average mileage at    *;
* time 1 should be similar for the 2 study groups.               *;                       
******************************************************************;

proc means n mean std min max maxdec=0 data=Columbo.all1;
  class study_group;
  var avg_ins; 
  title 'question 3';
  title2 'check for baseline imbalance';
run;


******************************************************************;
* Check whether baseline imbalance matters.                      *;
******************************************************************;

proc glm data=Columbo.all1;
  class study_group;
  model avg_miles = study_group / solution;
  lsmeans study_group / pdiff cl;
  title 'question 3';
  title2 'unadjusted comparison of the study groups (should match Shu)';
run;

proc glm data=Columbo.all1;
  class study_group;
  model avg_miles = study_group avg_ins / solution;
  lsmeans study_group / pdiff cl;
  title 'question 3';
  title2 'adjusting for baseline imbalance has no substantial impact';
run;



******************************************************************;
* Check annual mileage by car and overall                        *;
******************************************************************;

proc means n mean std min max maxdec=0 data=Columbo.all1;
  var car1_miles car2_miles car3_miles car4_miles avg_miles;
  title 'question 1';
  title2 'distribution of annual mileage by car and overall';
run;



******************************************************************;
* Annual miles per car are uncorrelated.                         *;
******************************************************************;

proc corr data=Columbo.all1;
  var car1_miles car2_miles car3_miles car4_miles;
  title 'correlations among annual mileage';
run;


******************************************************************;
* Histogram for annual mileage, car 1.                           *;
******************************************************************;

data temp;
  set Columbo.all1;
  car1_bin= 1 + int(car1_miles/2500);
run;


proc sgplot data=temp;
  vbar car1_bin; 
  title 'annual mileage, car 1, 20 bins of 2,500 miles each';
run;



******************************************************************;
* Check shapes of annual mileage by car.                         *;
* If X~Uniform(0,50000) then X/50000 ~ Uniform(0,1) and the      *;
* resulting QQ plot will have a 45 degree line.                  *;
******************************************************************;

data test_shape;
  set Columbo.all1;
  car1=car1_miles/50000;
  car2=car2_miles/50000;
  car3=car3_miles/50000;
  car4=car4_miles/50000;
run;

proc univariate data=test_shape;
  var car1;
  qqplot car1 / beta(alpha=1,beta=1);
  title 'QQ plot of miles driven, car 1, uniform distribution assumed';
  title2 '45 degree line = uniform distribution';
run;

proc univariate data=test_shape;
  var car2;
  qqplot car2 / beta(alpha=1,beta=1);
  title 'QQ plot of miles driven, car 2, uniform distribution assumed';
  title2 '45 degree line = uniform distribution';
run;

proc univariate data=test_shape;
  var car3;
  qqplot car3 / beta(alpha=1,beta=1);
  title 'QQ plot of miles driven, car 3, uniform distribution assumed';
  title2 '45 degree line = uniform distribution';
run;

proc univariate data=test_shape;
  var car4;
  qqplot car4 / beta(alpha=1,beta=1);
  title 'QQ plot of miles driven, car 4, uniform distribution assumed';
  title2 '45 degree line = uniform distribution';
run;

******************************************************************;
* Repeating the assessment of shape for car 1, by study group.   *;
******************************************************************;


proc univariate data=test_shape;
  where (study_group='Sign Top');
  var car1;
  qqplot car1 / beta(alpha=1,beta=1);
  title 'QQ plot of miles driven, car 1, uniform distribution assumed';
  title2 '45 degree line = uniform distribution';
  title3 'sign top';
run;

proc univariate data=test_shape;
  where (study_group='Sign Bottom');
  var car1;
  qqplot car1 / beta(alpha=1,beta=1);
  title 'QQ plot of miles driven, car 1, uniform distribution assumed';
  title2 '45 degree line = uniform distribution';
  title3 'sign bottom';
run;


******************************************************************;
* Consider mechanism.                                            *;
******************************************************************;


******************************************************************;
* Check end digit preference.                                    *;
******************************************************************;

data end_digit;
  set Columbo.all1;
  end1_ins=mod(car1_ins,10);
  end2_ins=mod(car2_ins,10);
  end3_ins=mod(car3_ins,10);
  end4_ins=mod(car4_ins,10);
  end1_rsp=mod(car1_rsp,10);
  end2_rsp=mod(car2_rsp,10);
  end3_rsp=mod(car3_rsp,10);
  end4_rsp=mod(car4_rsp,10);
run;

proc freq data=end_digit;
  tables end1_ins end2_ins end3_ins end4_ins;
  title 'end digit preference, time 1, by car, last digit';
run;


proc freq data=end_digit;
  tables end1_rsp end2_rsp end3_rsp end4_rsp;
  title 'end digit preference, time 2, by car, last digit';
run;


data end_digit;
  set Columbo.all1;
  end1_ins=mod(car1_ins,100);
  end2_ins=mod(car2_ins,100);
  end3_ins=mod(car3_ins,100);
  end4_ins=mod(car4_ins,100);
  end1_rsp=mod(car1_rsp,100);
  end2_rsp=mod(car2_rsp,100);
  end3_rsp=mod(car3_rsp,100);
  end4_rsp=mod(car4_rsp,100);
run;

proc freq data=end_digit;
  tables end1_ins end2_ins end3_ins end4_ins;
  title 'end digit preference, time 1, by car, last 2 digits';
run;


proc freq data=end_digit;
  tables end1_rsp end2_rsp end3_rsp end4_rsp;
  title 'end digit preference, time 2, by car, last 2 digits';
run;


data end_digit;
  set Columbo.all1;
  end1_ins=mod(car1_ins,1000);
  end2_ins=mod(car2_ins,1000);
  end3_ins=mod(car3_ins,1000);
  end4_ins=mod(car4_ins,1000);
  end1_rsp=mod(car1_rsp,1000);
  end2_rsp=mod(car2_rsp,1000);
  end3_rsp=mod(car3_rsp,1000);
  end4_rsp=mod(car4_rsp,1000);
run;


proc freq data=end_digit;
  tables end1_ins end2_ins end3_ins end4_ins;
  title 'end digit preference, time 1, by car, last 3 digits';
run;


proc freq data=end_digit;
  tables end1_rsp end2_rsp end3_rsp end4_rsp;
  title 'end digit preference, time 2, by car, last 3 digits';
run;



******************************************************************;
* Use fonts to explore the mechanism of data generation.         *;
******************************************************************;


******************************************************************;
* Check that the font counts are equal.                          *;
******************************************************************;

proc freq data=Columbo.all1;
  tables font;
  title 'font counts';
  title2 'these should be equal';
run;


******************************************************************;
* Check that the problem isn't an artifact of group assignment.  *;
******************************************************************;

proc freq data=Columbo.all1;
  tables study_group*font;
  title 'font counts';
  title2 'there are cambria records for both groups';
run;


******************************************************************;
* Check for end digit preference by font.                        *;
******************************************************************;


data end_digit;
  set Columbo.all1;
  end1_ins=mod(car1_ins,10);
  end2_ins=mod(car2_ins,10);
  end3_ins=mod(car3_ins,10);
  end4_ins=mod(car4_ins,10);
  end1_rsp=mod(car1_rsp,10);
  end2_rsp=mod(car2_rsp,10);
  end3_rsp=mod(car3_rsp,10);
  end4_rsp=mod(car4_rsp,10);
run;

proc freq data=end_digit;
  where font='Calibri';
  tables end1_ins;
  title 'end digit preference, car 1, year 1 (last digit)';
  title2 'Calibri font';
run;

proc freq data=end_digit;
  where font='Cambria';
  tables end1_ins;
  title 'end digit preference, car 1, year 1 (last digit)';
  title2 'Cambria font';
run;

proc freq data=end_digit;
  where font='Calibri';
  tables end1_rsp;
  title 'end digit preference, car 1, year 2 (last digit)';
  title2 'Calibri font';
run;

proc freq data=end_digit;
  where font='Cambria';
  tables end1_rsp;
  title 'end digit preference, car 1, year 2 (last digit)';
  title2 'Cambria font';
run;



data end_digit;
  set Columbo.all1a;
  end1_ins=mod(car1_ins,100);
  end2_ins=mod(car2_ins,100);
  end3_ins=mod(car3_ins,100);
  end4_ins=mod(car4_ins,100);
  end1_rsp=mod(car1_rsp,100);
  end2_rsp=mod(car2_rsp,100);
  end3_rsp=mod(car3_rsp,100);
  end4_rsp=mod(car4_rsp,100);
run;

proc freq data=end_digit;
  where font='Calibri';
  tables end1_ins;
  title 'end digit preference, car 1, year 1 (last 2 digits)';
  title2 'Calibri font';
run;

proc freq data=end_digit;
  where font='Cambria';
  tables end1_ins;
  title 'end digit preference, car 1, year 1 (last 2 digits)';
  title2 'Cambria font';
run;

proc freq data=end_digit;
  where font='Calibri';
  tables end1_rsp;
  title 'end digit preference, car 1, year 2 (last 2 digits)';
  title2 'Calibri font';
run;

proc freq data=end_digit;
  where font='Cambria';
  tables end1_rsp;
  title 'end digit preference, car 1, year 2 (last 2 digits)';
  title2 'Cambria font';
run;

data end_digit;
  set Columbo.all1;
  end1_ins=mod(car1_ins,1000);
  end2_ins=mod(car2_ins,1000);
  end3_ins=mod(car3_ins,1000);
  end4_ins=mod(car4_ins,1000);
  end1_rsp=mod(car1_rsp,1000);
  end2_rsp=mod(car2_rsp,1000);
  end3_rsp=mod(car3_rsp,1000);
  end4_rsp=mod(car4_rsp,1000);
run;

proc freq data=end_digit;
  where font='Calibri';
  tables end1_ins;
  title 'end digit preference, car 1, year 1 (last 3 digits)';
  title2 'Calibri font';
run;

proc freq data=end_digit;
  where font='Cambria';
  tables end1_ins;
  title 'end digit preference, car 1, year 1 (last 3 digits)';
  title2 'Cambria font';
run;

proc freq data=end_digit;
  where font='Calibri';
  tables end1_rsp;
  title 'end digit preference, car 1, year 2 (last 3 digits)';
  title2 'Calibri font';
run;

proc freq data=end_digit;
  where font='Cambria';
  tables end1_rsp;
  title 'end digit preference, car 1, year 2 (last 3 digits)';
  title2 'Cambria font';
run;



******************************************************************;
* Check for pairing of fonts.                                    *;
******************************************************************;


******************************************************************;
* Illustration of an edge case.                                  *;
******************************************************************;

 
data fonts; 
  set Columbo.all1;
  if (car4_ins ge 175000);
run;

proc sort data=fonts;
  by car4_ins;
run;

proc print data=fonts;
  var car4_ins font;
  title 'car 4 records, >= 175000 miles';
  title2 'check for pairing of fonts';
run;



******************************************************************;
* An unsophisticated attempt to match records.                   *;
******************************************************************;


data calibri(drop=font rename=(car1_ins=calibri)) cambria(drop=font rename=(car1_ins=cambria));
  set Columbo.all1(keep=font car1_ins);
  if (font='Calibri') then output calibri;
  else output cambria;
run;

proc sort data=calibri;
  by calibri;
run;

proc sort data=cambria;
  by cambria;
run;

data merged;
  merge calibri cambria;
  diff=cambria-calibri;
  if (0 le diff le 1000) then check='yes'; 
  else check='no';
run;

proc freq data=merged;
  tables check;
  title 'naive check on U(0,1000) hypothesis';
run;

proc means data=merged maxdec=0 min max mean std;
  var diff;
  title 'mean difference score';
run;




***************************************************************;
* Illustration of putative mechanism.                         *;
***************************************************************;


***************************************************************;
* Expected results from simulating Uniform (0,50000) random   *;
* variable.                                                   *;
***************************************************************;

data test;

call streaminit(1);
 
  do i=1 to 6700;
    y=rand("uniform",0,50000);
	output;
  end;

run;

proc means data=test n mean min max std data=test maxdec=0;
  var y;
  title 'expected simulation results, assuming uniform(0,50000)';
run;


***************************************************************;
* Expected simulation results, with adjustments at 2,500 and  *;
* 47,500 miles.                                               *;
***************************************************************;


data test2;
  set test;
  z=y;
  if ((i<3350) and (y>2500)) then z=y-2500;
run;


proc means n mean min max std data=test2 maxdec=0;
  var y z;
  title 'expected simulation results';
  title2 'sign bottom, adjustment at 2,500 miles';
run;

data test2;
  set test;
  z=y;
  if ((i<3350) and (y<47500)) then z=y+2500;
run;


proc means n mean min max std data=test2 maxdec=0;
  var y z;
  title 'expected simulation results';
  title2 'sign top, with adjustment at 47,500 miles';
run;


***************************************************************;
* Apply the simulation logic to the actual dataset.           *;
***************************************************************;

***************************************************************;
* Restructure the data to a long-thin structure, only keeping *;
* records with non-missing values for mileage.                *;
***************************************************************;

data long_thin(keep=id_var font study_group car t1);
  set Columbo.all1;
  car=1; t1=car1_ins; if (t1 ne .) then output;
  car=2; t1=car2_ins; if (t1 ne .) then output;
  car=3; t1=car3_ins; if (t1 ne .) then output;
  car=4; t1=car4_ins; if (t1 ne .) then output;
run;



***************************************************************;
* Create the two uniform random variables (one for annual     *;
* mileage, untransformed and one for the signal).             *;
***************************************************************; 

data long_thin2;
  set long_thin;
  call streaminit(10);
  u1=rand("uniform",0,50000);

  if (font='Cambria') then do;
     if (study_group='Sign Bottom') then do;
	    if (u1>2500) then t2t1=u1-2500;
		else t2t1=u1;
	 end;
	 if (study_group='Sign Top') then do;
	    if (u1<47500) then t2t1=u1+2500;
		else t2t1=u1;
	 end;
  end;

  if (font='Calibri') then t2t1=u1;

  t2=t2t1+t1;
run;

proc print data=long_thin2 (obs=20);
  title 'first 20 records, simulated dataset';
run;

proc means n mean std min max data=long_thin2;
  class font study_group;
  var t2t1;
  title 'means by font and study group';
run; 


***************************************************************;
* Return the dataset to its previous structure, now with      *;
* simulated values.                                           *;
***************************************************************; 

data car1 (keep=id_var study_group font car1_t1 car1_t2 car1_an)
     car2 (keep=id_var car2_t1 car2_t2 car2_an)
     car3 (keep=id_var car3_t1 car3_t2 car3_an)
     car4 (keep=id_var car4_t1 car4_t2 car4_an);
  set long_thin2;
 
   
   car1_t1=t1;
   car1_t2=t2;
   car1_an=t2t1;
 

   car2_t1=t1;
   car2_t2=t2;
   car2_an=t2t1;

   car3_t1=t1;
   car3_t2=t2;
   car3_an=t2t1;

   car4_t1=t1;
   car4_t2=t2;
   car4_an=t2t1;

   if car=1 then output car1;
   if car=2 then output car2;
   if car=3 then output car3;
   if car=4 then output car4;
run;

data short_fat;
  merge car1 car2 car3 car4;
  by id_var;

  avg_miles=mean(car1_an,car2_an,car3_an,car4_an);

run;

proc print data=short_fat(obs=10);
  title 'simulated data, first 10 records';
run;


***************************************************************;
* Calculate descriptive statistics on the simulated dataset.  *;
***************************************************************;

proc means data=short_fat n mean std min max maxdec=0;
  class font study_group;
  var avg_miles car1_an car2_an car3_an car4_an;
  title 'simulated means by font and study group';
run;


proc means data=short_fat n mean std min max maxdec=0;
  class study_group;
  var avg_miles;
  title 'simulated means by study group, primary outcome, simulated';
run;

proc means data=Columbo.all1 n mean std min max maxdec=0;
  class study_group;
  var avg_miles;
  title 'actual means by study group, primary outcome';
run;


